@@ -4,23 +4,51 @@ use warnings;
 use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.21';
+$VERSION = '0.30';
 
 #TODO: new accessors
 #wap
 #mobile
 #device
 
-sub name             { return shift->[UA_NAME]             || q{} }
-sub unknown          { return shift->[UA_UNKNOWN]          || q{} }
-sub generic          { return shift->[UA_GENERIC]          || q{} }
-sub os               { return shift->[UA_OS]               || q{} }
-sub lang             { return shift->[UA_LANG]             || q{} }
-sub strength         { return shift->[UA_STRENGTH]         || q{} }
-sub parser           { return shift->[UA_PARSER]           || q{} }
-sub original_name    { return shift->[UA_ORIGINAL_NAME]    || q{} }
-sub original_version { return shift->[UA_ORIGINAL_VERSION] || q{} }
-sub robot            { return shift->[UA_ROBOT]            ||   0 }
+BEGIN {
+    my @simple = qw(
+        name
+        unknown
+        generic
+        os
+        lang
+        strength
+        parser
+        original_name
+        original_version
+        robot
+    );
+
+    my @multi = qw(
+        mozilla
+        extras
+        dotnet
+    );
+
+    no strict qw(refs); ## no critic (TestingAndDebugging::ProhibitNoStrict)
+    foreach my $name ( @simple ) {
+        my $id = 'UA_' . uc $name;
+        $id = __PACKAGE__->$id();
+        *{ $name } = sub { return shift->[$id] || q{} };
+    }
+
+    foreach my $name ( @multi ) {
+        my $id = 'UA_' . uc $name;
+        $id = __PACKAGE__->$id();
+        *{ $name } = sub {
+            my $self = shift;
+            return +() if ! $self->[ $id ];
+            my @rv = @{ $self->[ $id ] };
+            return wantarray ? @rv : $rv[0];
+        };
+    }
+}
 
 sub version {
     my $self = shift;
@@ -28,29 +56,37 @@ sub version {
     return $self->[ $type eq 'raw' ? UA_VERSION_RAW : UA_VERSION ] || 0;
 }
 
-sub mozilla {
-    my $self = shift;
-    return +() if ! $self->[UA_MOZILLA];
-    my @rv = @{ $self->[UA_MOZILLA] };
-    return wantarray ? @rv : $rv[0];
-}
-
 sub toolkit {
     my $self = shift;
-    return +() if ! $self->[UA_TOOLKIT];
-    return @{ $self->[UA_TOOLKIT] };
+    return Parse::HTTP::UserAgent::Base::Accessors::toolkit->new(
+                $self->[UA_TOOLKIT]
+            );
 }
 
-sub extras {
-    my $self = shift;
-    return +() if ! $self->[UA_EXTRAS];
-    return @{ $self->[UA_EXTRAS] };
+package Parse::HTTP::UserAgent::Base::Accessors::toolkit;
+use strict;
+use warnings;
+use overload '""',    => 'name',
+             '0+',    => 'version',
+             fallback => 1,
+;
+use constant ID_NAME        => 0;
+use constant ID_VERSION_RAW => 1;
+use constant ID_VERSION     => 2;
+
+sub new {
+    my($class, $tk) = @_;
+    return bless [ $tk ? @{ $tk } : (undef) x 3 ], $class;
 }
 
-sub dotnet {
+sub name {
+    return shift->[ID_NAME];
+}
+
+sub version {
     my $self = shift;
-    return +() if ! $self->[UA_DOTNET];
-    return @{ $self->[UA_DOTNET] };
+    my $type = shift || q{};
+    return $self->[ $type eq 'raw' ? ID_VERSION_RAW : ID_VERSION ] || 0;
 }
 
 1;
@@ -74,8 +110,8 @@ Parse::HTTP::UserAgent::Base::Accessors - Available accessors
 
 =head1 DESCRIPTION
 
-This document describes version C<0.21> of C<Parse::HTTP::UserAgent::Base::Accessors>
-released on C<19 October 2011>.
+This document describes version C<0.30> of C<Parse::HTTP::UserAgent::Base::Accessors>
+released on C<27 October 2011>.
 
 Ther methods can be used to access the various parts of the parsed structure.
 
