@@ -5,7 +5,7 @@ use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 use constant OPERA_FAKER_EXTRA_SIZE => 4;
 
-$VERSION = '0.21';
+$VERSION = '0.30';
 
 sub _is_opera_pre {
     my($self, $moz) = @_;
@@ -27,6 +27,7 @@ sub _is_safari {
     my $str = $self->[UA_STRING];
     # epiphany?
     return                index( $str                   , 'Chrome'       ) != NO_IMATCH ? 0 # faker
+          :               index( $str                   , 'Android'      ) != NO_IMATCH ? 0 # faker
           :    $extra  && index( $extra->[0]            , 'AppleWebKit'  ) != NO_IMATCH ? 1
           : @{$others} && index( $others->[LAST_ELEMENT], 'Safari'       ) != NO_IMATCH ? 1
           :                                                                     0
@@ -44,6 +45,13 @@ sub _is_chrome {
            ( $extra  && index( $extra->[0], 'AppleWebKit') != NO_IMATCH);
 }
 
+sub _is_android {
+    my($self, $thing, $others) = @_;
+    my $has_android = grep { index( lc $_, 'android' ) != NO_IMATCH  } @{ $thing  };
+    my $has_safari  = grep { index( lc $_, 'safari'  ) != NO_IMATCH  } @{ $others };
+    return $has_android && $has_safari;
+}
+
 sub _is_ff {
     my($self, $extra) = @_;
     return if ! $extra || ! $extra->[1];
@@ -130,8 +138,8 @@ Parse::HTTP::UserAgent::Base::IS - Base class
 
 =head1 DESCRIPTION
 
-This document describes version C<0.21> of C<Parse::HTTP::UserAgent::Base::IS>
-released on C<19 October 2011>.
+This document describes version C<0.30> of C<Parse::HTTP::UserAgent::Base::IS>
+released on C<27 October 2011>.
 
 Internal module.
 
